@@ -13,18 +13,43 @@ from abstract_utilities import (get_date,
                                 safe_json_loads,
                                 safe_dump_to_file,
                                 read_from_file,
-                                write_to_file)
+                                write_to_file,
+                                is_number,)
+def is_bool(obj,convert=False):
+    converted = obj
+    is_bool = False
+    if obj in [0,False,'false','False','F']:
+        converted = bool(obj)
+        is_bool = True
+    if obj in  [1,True,'true','True','T']:
+        converted = bool(obj)
+        is_bool = True
+    if convert:
+        return converted
+    return is_bool
+        
 class SaveManager:
     """
     Manages the saving of data. This class should provide methods to specify where (e.g., what database or file) and how (e.g., in what format) data should be saved.
     """
-    def __init__(self, data={},title=None,directory=None,model='default'):
+    def __init__(self, data={},title:str=None,directory:str=None,model:str='default')->None:
         self.title=title
         self.model=model
         self.date = get_date()
-        self.directory = mkdirs(directory or os.path.join(os.getcwd(), 'response_data'))
-        self.directory = mkdirs(os.path.join(self.directory, self.date))
-        self.directory = mkdirs(os.path.join(self.directory, self.model))
+        if isinstance(directory,list):
+            abs_path = directory[0]
+            if not os.path.isabs(directory[0]):
+                abs_path = os.path.join(os.getcwd(), directory[0])
+                mkdirs(abs_path)
+            path = abs_path
+            for child in directory[1:]:
+                path = os.path.join(path, child)
+                mkdirs(path)
+            self.directory=path
+        else:
+            self.directory = mkdirs(directory or os.path.join(os.getcwd(), 'response_data'))
+            self.directory = mkdirs(os.path.join(self.directory, self.date))
+            self.directory = mkdirs(os.path.join(self.directory, self.model))
         self.file_name = self.create_unique_file_name()
         self.file_path = os.path.join(self.directory, self.file_name)
         if data:
@@ -33,7 +58,7 @@ class SaveManager:
             self.data['title']=self.title
             self.data['model']=self.model
             self.save_to_file(data = data,file_path = self.file_path)
-    def create_unique_file_name(self):
+    def create_unique_file_name(self)->str:
         base_name = f"{self.title}.json"
         index = 1
         unique_name = base_name
@@ -41,13 +66,13 @@ class SaveManager:
             unique_name = f"{self.title}_{index}.json"
             index += 1
         return unique_name
-    def save_to_file(self, data, file_path):
+    def save_to_file(self, data:dict, file_path:str)->None:
         # Assuming `data` is already a dictionary, we convert it to a JSON string and save.
         with open(file_path, 'w', encoding='utf-8') as file:
             json.dump(data, file, ensure_ascii=False, indent=4)
     
     @staticmethod
-    def read_saved_json(file_path):
+    def read_saved_json(file_path:str)->dict:
         # Use 'safe_read_from_json' which is presumed to handle errors and return JSON
         return safe_read_from_json(file_path)
 class ResponseManager:
@@ -86,7 +111,7 @@ class ResponseManager:
         initial_query: Manages the initial sequence of sending queries and processing responses.
 
     """
-    def __init__(self,prompt_mgr=None,api_mgr=None,title=None,directory=None):
+    def __init__(self,prompt_mgr=None,api_mgr=None,title:str=None,directory:str=None)->None:
         self.prompt_mgr=prompt_mgr
         self.model_mgr=self.prompt_mgr.model_mgr
         self.api_mgr=api_mgr
@@ -102,7 +127,7 @@ class ResponseManager:
         self.original_title=self.title
         self.re_initialize_query()
         self.response_keys = ["api_response","abort","additional_responses","suggestions","notation","generate_title","request_chunks","token_adjustment","prompt_as_previous","created","model","error",'content']
-    def re_initialize_query(self):
+    def re_initialize_query(self)->None:
         self.i_query=0
         self.generate_title=None
         self.query_done=False
@@ -113,7 +138,7 @@ class ResponseManager:
         self.request_chunks=None
         self.token_adjustment=None
         self.prompt_as_previous_clone=None
-    def post_request(self):
+    def post_request(self)->(str or dict):
         """
         Sends a POST request to the specified endpoint with the provided prompt and headers.
         
@@ -136,59 +161,89 @@ class ResponseManager:
     def get_response(self):
         # Convert the response to JSON directly within this method
         try:
-            self.api_response = self.response.json()
+            self.current_response = self.response.json()
         except ValueError:
             # Handle the case where the response is not JSON
-            self.api_response = {"error": "Response is not in JSON format", "data": self.response.text}
+            self.current_response = {"error": "Response is not in JSON format", "data": self.response.text}
         # Save the response using SaveManager
-        json_api_response = safe_json_loads(self.api_response)
-        self.get_all_instruction(json_api_response)
-        self.query_js["response"]=json_api_response
+        self.query_js['response']=safe_json_loads(self.current_response)
+        
+        self.save_manager = SaveManager(data=self.query_js['response'], title=self.query_js['response'].get('created','response'), directory=['response_data','raw_response'], model=self.query_js['response'].get('model','default'))
+        self.current_content = safe_json_loads(get_any_value(self.query_js['response'],'content'))
+        if self.current_content and isinstance(self.current_content,list):
+            self.current_content=safe_json_loads(self.current_content[0])
+        self.error = None
+        self.error_response = get_any_value(self.query_js['response'],'error')
+        if self.error_response:
+            self.error = 'error'
+        self.model=get_any_value(self.query_js['response'],"model")
+        self.created=get_any_value(self.query_js['response'],"created")
+        self.model = self.model or self.error or 'default'
+        self.get_all_instruction(data=self.current_content)
+        self.title = self.generate_title or self.original_title or self.created
         self.save_manager = SaveManager(data=self.query_js, title=self.generate_title or self.title or 'response', directory=self.directory, model=self.model)
         self.output.append(self.save_manager.data)
-    def try_load_response(self):
+    def try_load_response(self)->None:
         """
         Attempts to load the response content into a structured format.
         """
         self.api_response = safe_json_loads(self.get_response())
-    def extract_response(self):
+    def extract_response(self)->dict:
         """
         Processes the response and manages the creation of a save point through `SaveManager`.
         """
         return self.query_js
-    def get_abort(self):
-       self.abort_js = {"abort":None,"additional_responses":None,"request_chunks":None,"blank_prompt":self.prompt.get('blank_prompt')}
-       for key,value in self.prompt_mgr.instruction_mgr.instructions_js.items():
-           if key in list(self.abort_js.keys()):
-              setattr(self,key,value)
-              self.abort_js[key]=value
-           if self.abort_js["abort"] or self.abort_js["blank_prompt"]:
-               return True
+    def get_abort(self)->bool:
+       if self.abort_js["abort"] or self.abort_js["blank_prompt"]:
+            return True
        if self.abort_js["request_chunks"] or self.abort_js["additional_responses"]:
            return False
        else:
            return True
-    def get_all_instruction(self,data):
-        self.prompt_as_previous_clone
-        if data:
-            for i,key in enumerate(self.response_keys):
-                value = get_any_value(data,key)
-                if isinstance(value,list) and value:
-                    value=value[-1]
-                setattr(self,key,value)
-                if key == 'prompt_as_previous' and value:
-                    self.prompt_as_previous_clone=self.api_response
-            error_response = get_any_value(data,'error')
-            if error_response:
-                self.error = 'error'
-            self.model = self.model or self.error or 'default'
-            self.title = self.generate_title or self.original_title or self.created
-            self.get_abort()
-    def send_query(self):
+    def get_all_instruction(self,data:dict)->None:
+        self.current_api_response=data.get('api_response','')
+        self.suggestions,self.notation,self.generate_title,self.prompt_as_previous_clone,self.additional_responses,self.abort,self.request_chunks,self.token_adjustment=None,None,self.generate_title,None,None,None,None,None,None,0
+        self.abort_js = {"abort":None,"additional_responses":None,"request_chunks":None}
+        text_response_keys = ["suggestions","notation","generate_title"]
+        bool_response_keys =["prompt_as_previous","additional_responses","abort","request_chunks"]
+        int_response_keys = ["token_adjustment"]
+        for response_key,value in data.items():
+            if is_bool(value):
+                value = bool(value)
+            elif is_number(value):
+                value = int(value)
+            elif value == '':
+                value = None
+            else:
+                value = str(value)
+            if response_key in self.abort_js.keys():
+                self.abort_js[response_key]=bool(value)
+            
+            elif response_key in text_response_keys:
+                if response_key == "notation":
+                    self.notation = value
+                elif response_key == "generate_title":
+                    self.generate_title = value
+            elif response_key in bool_response_keys:
+                if response_key == "prompt_as_previous":
+                    if bool(value):
+                        self.prompt_as_previous_clone = self.current_api_response
+                elif response_key == "additional_responses":
+                    self.additional_responses = value
+                elif response_key == "abort":
+                    self.abort = value
+                elif request_key == "request_chunks":
+                    self.request_chunks = value
+            elif response_key == "token_adjustment":
+                self.token_adjustment == int(value)
+        
+    
+    def send_query(self)->None:
         """
         Handles the response after a query has been sent.
         """
         self.query_js={}
+        input(self.prompt_as_previous_clone)
         self.prompt = self.prompt_mgr.create_prompt(chunk_token_distribution_number=self.chunk_token_distribution_number,
                                                     chunk_number = self.chunk_number,
                                                     notation=self.notation,
@@ -202,7 +257,7 @@ class ResponseManager:
             self.header = self.api_mgr.header
             self.response = requests.post(url=self.endpoint, json=self.prompt, headers=self.header)
             safe_json_loads(self.get_response())
-    def initial_query(self):
+    def initial_query(self)->list[dict]:
         """
         Manages the initial sequence of sending queries and processing responses.
 
